@@ -48,6 +48,7 @@ class LexerDefinition(object):
         'NE',
         'LITERAL',
         'CURRENT',
+        'EXPREF',
     ) + tuple(reserved.values())
 
     t_STAR = r'\*'
@@ -69,6 +70,7 @@ class LexerDefinition(object):
     t_EQ = r'=='
     t_NE = r'!='
     t_CURRENT = r'@'
+    t_EXPREF = r'&'
     t_ignore = ' '
 
     def t_NUMBER(self, t):
@@ -150,4 +152,6 @@ class LexerDefinition(object):
                 lexer_value=t.value,
                 message=("Bad token '%s': starting quote is missing "
                          "the ending quote" % t.value))
-        raise ValueError("Illegal token value '%s'" % t.value)
+        raise LexerError(lexer_position=t.lexpos,
+                         lexer_value=t.value,
+                         message=("Illegal token value '%s'" % t.value))
