@@ -5,6 +5,7 @@ from tests import unittest
 from jmespath import parser
 from jmespath import ast
 from jmespath import lexer
+from jmespath import compat
 
 
 class TestParser(unittest.TestCase):
@@ -32,9 +33,9 @@ class TestParser(unittest.TestCase):
 
     def test_quoted_subexpression(self):
         parsed = self.parser.parse('"foo"."bar"')
-        self.assertIsInstance(parsed, ast.SubExpression)
-        self.assertEqual(parsed.parent.name, 'foo')
-        self.assertEqual(parsed.child.name, 'bar')
+        self.assertIsInstance(parsed.parsed, ast.SubExpression)
+        self.assertEqual(parsed.parsed.parent.name, 'foo')
+        self.assertEqual(parsed.parsed.child.name, 'bar')
 
     def test_wildcard(self):
         parsed = self.parser.parse('foo[*]')
@@ -65,6 +66,17 @@ class TestParser(unittest.TestCase):
         parsed = self.parser.parse('foo || bar')
         self.assertEqual(repr(parsed), 'ORExpression(Field(foo), Field(bar))')
 
+    def test_unicode_literals_escaped(self):
+        parsed = self.parser.parse(r'`"\u2713"`')
+        if compat.PY2:
+            self.assertEqual(repr(parsed), r'Literal(\u2713)')
+        else:
+            self.assertEqual(repr(parsed), u'Literal(\u2713)')
+
+    def test_unicode_pretty_print(self):
+        parsed = self.parser.parse(r'`"\u2713"`')
+        self.assertEqual(parsed.pretty_print(), u'Literal(\u2713)')
+
     def test_multiselect(self):
         parsed = self.parser.parse('foo.{bar: bar,baz: baz}')
         self.assertEqual(
@@ -132,6 +144,19 @@ class TestErrorMessages(unittest.TestCase):
         self.assert_error_message('foo."bar', error_message,
                                   exception=lexer.LexerError)
 
+    def test_bad_lexer_literal_value_with_json_object(self):
+        error_message = ('Bad jmespath expression: '
+                         'Bad token `{{}`:\n`{{}`\n^')
+        self.assert_error_message('`{{}`', error_message,
+                                  exception=lexer.LexerError)
+
+
+    def test_bad_unicode_string(self):
+        error_message = ('Bad jmespath expression: '
+                         'Invalid \\uXXXX escape:\n"\\uAZ12"\n^')
+        self.assert_error_message(r'"\uAZ12"', error_message,
+                                  exception=lexer.LexerError)
+
 
 class TestParserWildcards(unittest.TestCase):
     def setUp(self):
@@ -183,22 +208,22 @@ class TestParserWildcards(unittest.TestCase):
         self.assertEqual(sorted(parsed.search(data)), sorted(['a', 'b', 'c']))
 
     def test_escape_sequences(self):
-        self.assertEqual(self.parser.parse('"foo\tbar"').search(
+        self.assertEqual(self.parser.parse(r'"foo\tbar"').search(
             {'foo\tbar': 'baz'}), 'baz')
-        self.assertEqual(self.parser.parse('"foo\nbar"').search(
+        self.assertEqual(self.parser.parse(r'"foo\nbar"').search(
             {'foo\nbar': 'baz'}), 'baz')
-        self.assertEqual(self.parser.parse('"foo\bbar"').search(
+        self.assertEqual(self.parser.parse(r'"foo\bbar"').search(
             {'foo\bbar': 'baz'}), 'baz')
-        self.assertEqual(self.parser.parse('"foo\fbar"').search(
+        self.assertEqual(self.parser.parse(r'"foo\fbar"').search(
             {'foo\fbar': 'baz'}), 'baz')
-        self.assertEqual(self.parser.parse('"foo\rbar"').search(
+        self.assertEqual(self.parser.parse(r'"foo\rbar"').search(
             {'foo\rbar': 'baz'}), 'baz')
 
     def test_consecutive_escape_sequences(self):
-        parsed = self.parser.parse('"foo\\nbar"')
+        parsed = self.parser.parse(r'"foo\\nbar"')
         self.assertEqual(parsed.search({'foo\\nbar': 'baz'}), 'baz')
 
-        parsed = self.parser.parse('"foo\n\t\rbar"')
+        parsed = self.parser.parse(r'"foo\n\t\rbar"')
         self.assertEqual(parsed.search({'foo\n\t\rbar': 'baz'}), 'baz')
 
     def test_escape_sequence_at_end_of_string_not_allowed(self):
@@ -267,8 +292,16 @@ class TestParserCaching(unittest.TestCase):
         # cache but they should still be equal to compiled.
         for i in range(parser.Parser._max_size + 1):
             compiled2.append(p.parse('foo%s' % i))
+        self.assertEqual(len(compiled), len(compiled2))
         self.assertEqual(compiled, compiled2)
 
 
+class TestParserAddsExpressionAttribute(unittest.TestCase):
+    def test_expression_available_from_parser(self):
+        p = parser.Parser()
+        parsed = p.parse('foo.bar')
+        self.assertEqual(parsed.expression, 'foo.bar')
+
+
 if __name__ == '__main__':
     unittest.main()
